@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2012 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2014 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -543,7 +543,7 @@ public class AJPv13Response {
         for (int i = 0; i < formattedCookies.length; i++) {
             final int hdrNameLen;
             {
-                final String hdrName = i == 0 ? STR_SET_COOKIE : new com.openexchange.java.StringAllocator(STR_SET_COOKIE.length() + 1).append(STR_SET_COOKIE).append(
+                final String hdrName = i == 0 ? STR_SET_COOKIE : new StringBuilder(STR_SET_COOKIE.length() + 1).append(STR_SET_COOKIE).append(
                     i + 1).toString();
                 /*
                  * Set-Cookie and Set-Cookie2 is encoded in AJP protocol as an integer value
