@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2012 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2014 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -56,6 +56,7 @@ import java.nio.charset.Charset;
 import java.security.Principal;
 import java.security.SecureRandom;
 import java.text.ParseException;
+import java.util.Collection;
 import java.util.Collections;
 import java.util.Enumeration;
 import java.util.HashMap;
@@ -66,15 +67,22 @@ import java.util.Locale;
 import java.util.Map;
 import java.util.Map.Entry;
 import java.util.Set;
+import javax.servlet.AsyncContext;
+import javax.servlet.DispatcherType;
 import javax.servlet.RequestDispatcher;
 import javax.servlet.ServletContext;
+import javax.servlet.ServletException;
 import javax.servlet.ServletInputStream;
+import javax.servlet.ServletRequest;
+import javax.servlet.ServletResponse;
 import javax.servlet.http.Cookie;
 import javax.servlet.http.HttpServlet;
 import javax.servlet.http.HttpServletRequest;
+import javax.servlet.http.HttpServletResponse;
 import javax.servlet.http.HttpSession;
+import javax.servlet.http.Part;
 import com.openexchange.ajp13.AJPv13Config;
-import com.openexchange.ajp13.Services;
+import com.openexchange.ajp13.AJPv13ServiceRegistry;
 import com.openexchange.ajp13.exception.AJPv13Exception;
 import com.openexchange.ajp13.exception.AJPv13Exception.AJPCode;
 import com.openexchange.ajp13.servlet.ServletConfigLoader;
@@ -85,11 +93,11 @@ import com.openexchange.config.ConfigTools;
 import com.openexchange.config.ConfigurationService;
 import com.openexchange.configuration.ServerConfig;
 import com.openexchange.configuration.ServerConfig.Property;
-import com.openexchange.dispatcher.Parameterizable;
 import com.openexchange.exception.OXException;
 import com.openexchange.java.DefaultHashKeyGenerator;
 import com.openexchange.java.HashKeyGenerator;
 import com.openexchange.java.HashKeyMap;
+import com.openexchange.log.Log;
 import com.openexchange.mail.mime.ContentType;
 import com.openexchange.tools.servlet.http.Tools;
 
@@ -98,7 +106,7 @@ import com.openexchange.tools.servlet.http.Tools;
  *
  * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
  */
-public final class HttpServletRequestImpl implements HttpServletRequest, Parameterizable {
+public final class HttpServletRequestImpl implements HttpServletRequest {
 
     private static final org.slf4j.Logger LOG = org.slf4j.LoggerFactory.getLogger(HttpServletRequestImpl.class);
 
@@ -181,7 +189,7 @@ public final class HttpServletRequestImpl implements HttpServletRequest, Paramet
 
     private HttpServlet servletInstance;
 
-    private volatile long startTime;
+    private long startTime;
 
     private ActionAwareServletInputStream servletInputStream;
 
@@ -206,7 +214,6 @@ public final class HttpServletRequestImpl implements HttpServletRequest, Paramet
      */
     public HttpServletRequestImpl(final HttpServletResponseImpl response) {
         super();
-        contentLength = -1L;
         max = AJPv13Config.getMaxRequestParameterCount();
         contextPath = "";
         requestedSessionIdFromCookie = true;
@@ -220,11 +227,10 @@ public final class HttpServletRequestImpl implements HttpServletRequest, Paramet
         parameters = new HashKeyMap<List<String>>(max > 0 ? max : 64).setGenerator(hashKeyGenerator);
         headers = new HashMap<String, List<String>>(16);
         try {
-            setHeaderInternal(CONTENT_LENGTH, Integer.toString(-1), false);
+            setHeaderInternal(CONTENT_LENGTH, String.valueOf(-1), false);
         } catch (final AJPv13Exception e) {
             // Cannot occur
         }
-        startTime = System.currentTimeMillis();
     }
 
     /**
@@ -245,15 +251,6 @@ public final class HttpServletRequestImpl implements HttpServletRequest, Paramet
         servletInputStream.dumpToBuffer(bytes);
     }
 
-    /**
-     * Append specified bytes to buffer.
-     *
-     * @param bytes The bytes
-     */
-    public void appendToBuffer(final byte[] bytes) {
-        servletInputStream.appendToBuffer(bytes);
-    }
-
     /**
      * Checks if this request's content type indicates the form data: <code>"application/x-www-form-urlencoded"</code>
      *
@@ -276,7 +273,7 @@ public final class HttpServletRequestImpl implements HttpServletRequest, Paramet
         parameters.setGenerator(hashKeyGenerator);
         headers.clear();
         try {
-            setHeaderInternal(CONTENT_LENGTH, Integer.toString(-1), false);
+            setHeaderInternal(CONTENT_LENGTH, String.valueOf(-1), false);
         } catch (final AJPv13Exception e) {
             // Cannot occur
         }
@@ -309,7 +306,7 @@ public final class HttpServletRequestImpl implements HttpServletRequest, Paramet
         requestedSessionIdFromCookie = true;
         requestedSessionIdFromURL = false;
         servletInstance = null;
-        startTime = System.currentTimeMillis();
+        startTime = 0L;
         contentLength = -1L;
     }
 
@@ -320,7 +317,7 @@ public final class HttpServletRequestImpl implements HttpServletRequest, Paramet
      */
     public void setContentLength(final long contentLength) {
         try {
-            setHeaderInternal(CONTENT_LENGTH, Long.toString(contentLength), false);
+            setHeaderInternal(CONTENT_LENGTH, String.valueOf(contentLength), false);
         } catch (final AJPv13Exception e) {
             // Cannot occur
         }
@@ -354,20 +351,8 @@ public final class HttpServletRequestImpl implements HttpServletRequest, Paramet
         }
     }
 
-    @Override
-    public void putParameter(String name, String value) {
-        if (null == name) {
-            throw new NullPointerException("name is null");
-        }
-        if (null == value) {
-            parameters.remove(name);
-        } else {
-            setParameter(name, value);
-        }
-    }
-
-    private static List<String> newLinkedList(final String initialValue) {
-        final List<String> list = new LinkedList<String>();
+    private static LinkedList<String> newLinkedList(final String initialValue) {
+        final LinkedList<String> list = new LinkedList<String>();
         list.add(initialValue);
         return list;
     }
@@ -460,7 +445,8 @@ public final class HttpServletRequestImpl implements HttpServletRequest, Paramet
      */
     @Override
     public String getHeader(final String name) {
-        return makeString(headers.get(name.toLowerCase(Locale.ENGLISH)));
+        final String n = name.toLowerCase(Locale.ENGLISH);
+        return makeString(headers.get(n));
     }
 
     /**
@@ -480,7 +466,7 @@ public final class HttpServletRequestImpl implements HttpServletRequest, Paramet
      * @return The header values as an {@link Enumeration}
      */
     @Override
-    public Enumeration<?> getHeaders(final String name) {
+    public Enumeration<String> getHeaders(final String name) {
         return makeEnumeration(headers.get(name.toLowerCase(Locale.ENGLISH)));
     }
 
@@ -490,7 +476,7 @@ public final class HttpServletRequestImpl implements HttpServletRequest, Paramet
      * @return The header names as an {@link Enumeration}
      */
     @Override
-    public Enumeration<?> getHeaderNames() {
+    public Enumeration<String> getHeaderNames() {
         return makeEnumeration(headers.keySet().iterator());
     }
 
@@ -510,12 +496,12 @@ public final class HttpServletRequestImpl implements HttpServletRequest, Paramet
     }
 
     @Override
-    public Enumeration<?> getParameterNames() {
+    public Enumeration<String> getParameterNames() {
         return makeEnumeration(parameters.keySet().iterator());
     }
 
     @Override
-    public Map<?, ?> getParameterMap() {
+    public Map<String, String[]> getParameterMap() {
         final Map<String, String[]> retval = new HashKeyMap<String[]>(parameters.size()).setGenerator(hashKeyGenerator);
         for (final Entry<String, List<String>> entry : parameters.entrySet()) {
             final List<String> values = entry.getValue();
@@ -540,7 +526,7 @@ public final class HttpServletRequestImpl implements HttpServletRequest, Paramet
     }
 
     @Override
-    public Enumeration<?> getAttributeNames() {
+    public Enumeration<String> getAttributeNames() {
         return makeEnumeration(attributes.keySet().iterator());
     }
 
@@ -569,10 +555,6 @@ public final class HttpServletRequestImpl implements HttpServletRequest, Paramet
     @Override
     public void setCharacterEncoding(final String characterEncoding) throws UnsupportedEncodingException {
         String charset = characterEncoding;
-        if (null == charset) {
-            charset = "ISO-8859-1";
-            return;
-        }
         final int mlen;
         if (charset.charAt(0) == '"' && charset.charAt((mlen = charset.length() - 1)) == '"') {
             charset = charset.substring(1, mlen);
@@ -605,7 +587,7 @@ public final class HttpServletRequestImpl implements HttpServletRequest, Paramet
     }
 
     @Override
-    public Enumeration<?> getLocales() {
+    public Enumeration<Locale> getLocales() {
         return null;
     }
 
@@ -693,7 +675,7 @@ public final class HttpServletRequestImpl implements HttpServletRequest, Paramet
              * Determine scheme from protocol (in the form protocol/majorVersion.minorVersion) and isSecure information
              */
             scheme =
-                new com.openexchange.java.StringAllocator(protocol.substring(0, protocol.indexOf('/')).toLowerCase(Locale.ENGLISH)).append(secure ? "s" : "").toString();
+                new StringBuilder(protocol.substring(0, protocol.indexOf('/')).toLowerCase(Locale.ENGLISH)).append(secure ? "s" : "").toString();
         }
         return scheme;
     }
@@ -776,20 +758,6 @@ public final class HttpServletRequestImpl implements HttpServletRequest, Paramet
         return values.get(0);
     }
 
-    private static final Enumeration EMPTY_ENUM = new Enumeration() {
-
-        @Override
-        public boolean hasMoreElements() {
-            return false;
-        }
-
-        @Override
-        public Object nextElement() {
-            return null;
-        }
-
-    };
-
     /**
      * Creates a new {@link Enumeration} for specified array.
      *
@@ -798,9 +766,6 @@ public final class HttpServletRequestImpl implements HttpServletRequest, Paramet
      * @return A new {@link Enumeration}
      */
     protected static <T> Enumeration<T> makeEnumeration(final List<T> list) {
-        if (null == list) {
-            return EMPTY_ENUM;
-        }
         return (new Enumeration<T>() {
 
             private final int size = list.size();
@@ -930,12 +895,7 @@ public final class HttpServletRequestImpl implements HttpServletRequest, Paramet
     }
 
     public void setPathInfo(final String pathInfo) {
-        if (null == pathInfo) {
-            this.pathInfo = null;
-        } else {
-            // Ensure starting slash '/' character
-            this.pathInfo = pathInfo.length() > 0 && '/' != pathInfo.charAt(0) ? '/' + pathInfo : pathInfo;
-        }
+        this.pathInfo = pathInfo;
     }
 
     @Override
@@ -999,7 +959,9 @@ public final class HttpServletRequestImpl implements HttpServletRequest, Paramet
 
     @Override
     public boolean isUserInRole(final String role) {
-        LOG.warn("Method isUserInRole() is not implemented in HttpServletRequestWrapper, yet!");
+        if (LOG.isWarnEnabled()) {
+            LOG.warn("Method isUserInRole() is not implemented in HttpServletRequestWrapper, yet!");
+        }
         return false;
     }
 
@@ -1084,8 +1046,8 @@ public final class HttpServletRequestImpl implements HttpServletRequest, Paramet
          * First look-up HttpSessionManagement if a session already exists
          */
         final AjpProcessor ajpProcessor = response.getAjpProcessor();
-        final Cookie jsessionIdCookie = ajpProcessor.getHttpSessionCookie();
-        final String httpSessionId = jsessionIdCookie.getValue();
+        final Cookie sessionCookie = ajpProcessor.getHttpSessionCookie();
+        final String httpSessionId = sessionCookie.getValue();
         final HttpSessionWrapper httpSession = HttpSessionManagement.getHttpSession(httpSessionId);
         if (httpSession != null) {
             if (!HttpSessionManagement.isHttpSessionExpired(httpSession)) {
@@ -1106,64 +1068,38 @@ public final class HttpServletRequestImpl implements HttpServletRequest, Paramet
             HttpSessionManagement.removeHttpSession(httpSessionId);
         }
         /*
-         * Check whether to create a new session
+         * Create a new session
          */
         if (create) {
+            /*
+             * Create new session
+             */
             session = ((HttpSessionWrapper) HttpSessionManagement.createAndGetHttpSession(httpSessionId));
             session.setNew(true);
             session.setServletContext(getServletContext());
             /*
              * Add JSESSIONID cookie
              */
-            configureCookie(jsessionIdCookie);
-            response.addCookie(jsessionIdCookie);
+            configureCookie(sessionCookie);
+            response.addCookie(sessionCookie);
         }
         return session;
     }
 
-    private static volatile Boolean autologin;
+    private static final String DEFAULT_PATH = "/";
 
-    private static boolean autologin() {
-        Boolean tmp = autologin;
-        if (null == tmp) {
-            synchronized (HttpServletResponseImpl.class) {
-                tmp = autologin;
-                if (null == tmp) {
-                    final ConfigurationService configurationService = Services.getService(ConfigurationService.class);
-                    tmp = Boolean.valueOf(null != configurationService && configurationService.getBoolProperty("com.openexchange.sessiond.autologin", false));
-                    autologin = tmp;
-                }
-            }
-        }
-        return tmp.booleanValue();
-    }
-
-    private static volatile Integer maxAge;
-
-    private static int maxAge() {
-        Integer tmp = maxAge;
-        if (null == tmp) {
-            synchronized (HttpServletResponseImpl.class) {
-                tmp = maxAge;
-                if (null == tmp) {
-                    final ConfigurationService service = Services.getService(ConfigurationService.class);
-                    tmp = Integer.valueOf(ConfigTools.parseTimespanSecs(null == service ?  "1W" : service.getProperty("com.openexchange.cookie.ttl", "1W")));
-                    maxAge = tmp;
-                }
-            }
-        }
-        return tmp.intValue();
-    }
-
-    public static void configureCookie(final Cookie jsessionIdCookie) {
+    private static void configureCookie(final Cookie sessionCookie) {
+        sessionCookie.setPath(DEFAULT_PATH);
+        final ConfigurationService configurationService = AJPv13ServiceRegistry.getInstance().getService(ConfigurationService.class);
         /*
          * Check if auto-login is enabled
          */
-        if (autologin()) {
-            final int maxAge = maxAge();
-            jsessionIdCookie.setMaxAge(maxAge);
+        if (null != configurationService && configurationService.getBoolProperty("com.openexchange.sessiond.autologin", false)) {
+            final int maxAge =
+                (int) (ConfigTools.parseTimespan(configurationService.getProperty("com.openexchange.cookie.ttl", "1W")) / 1000);
+            sessionCookie.setMaxAge(maxAge);
         } else {
-            jsessionIdCookie.setMaxAge(-1); // cookies auto-expire
+            sessionCookie.setMaxAge(-1); // cookies auto-expire
         }
     }
 
@@ -1217,7 +1153,8 @@ public final class HttpServletRequestImpl implements HttpServletRequest, Paramet
         }
     }
 
-    private final ServletContext getServletContext() {
+    @Override
+    public ServletContext getServletContext() {
         return ServletConfigLoader.getDefaultInstance().getContext(servletInstance.getClass().getCanonicalName(), servletPath);
     }
 
@@ -1239,4 +1176,59 @@ public final class HttpServletRequestImpl implements HttpServletRequest, Paramet
         return startTime;
     }
 
+    @Override
+    public AsyncContext startAsync() throws IllegalStateException {
+        return null;
+    }
+
+    @Override
+    public AsyncContext startAsync(final ServletRequest servletRequest, final ServletResponse servletResponse) throws IllegalStateException {
+        return null;
+    }
+
+    @Override
+    public boolean isAsyncStarted() {
+        return false;
+    }
+
+    @Override
+    public boolean isAsyncSupported() {
+        return false;
+    }
+
+    @Override
+    public AsyncContext getAsyncContext() {
+        return null;
+    }
+
+    @Override
+    public DispatcherType getDispatcherType() {
+        return null;
+    }
+
+    @Override
+    public boolean authenticate(final HttpServletResponse response) throws IOException, ServletException {
+        return false;
+    }
+
+    @Override
+    public void login(final String username, final String password) throws ServletException {
+        // Empty
+    }
+
+    @Override
+    public void logout() throws ServletException {
+        // Empty
+    }
+
+    @Override
+    public Collection<Part> getParts() throws IOException, ServletException {
+        return Collections.emptyList();
+    }
+
+    @Override
+    public Part getPart(final String name) throws IOException, ServletException {
+        return null;
+    }
+
 }
