@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2012 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2014 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -49,15 +49,20 @@
 
 package com.openexchange.ajp13.servlet.http;
 
+import java.io.IOException;
 import java.security.Principal;
 import java.text.ParseException;
+import java.util.Collection;
 import javax.servlet.ServletContext;
+import javax.servlet.ServletException;
 import javax.servlet.http.Cookie;
 import javax.servlet.http.HttpServlet;
 import javax.servlet.http.HttpServletRequest;
+import javax.servlet.http.HttpServletResponse;
 import javax.servlet.http.HttpSession;
+import javax.servlet.http.Part;
 import com.openexchange.ajp13.AJPv13RequestHandler;
-import com.openexchange.ajp13.Services;
+import com.openexchange.ajp13.AJPv13ServiceRegistry;
 import com.openexchange.ajp13.exception.AJPv13Exception;
 import com.openexchange.ajp13.servlet.ServletConfigLoader;
 import com.openexchange.ajp13.servlet.ServletRequestWrapper;
@@ -206,7 +211,9 @@ public class HttpServletRequestWrapper extends ServletRequestWrapper implements
 
     @Override
     public boolean isUserInRole(final String role) {
-        LOG.warn("Method isUserInRole() is not implemented in HttpServletRequestWrapper, yet!");
+        if (LOG.isWarnEnabled()) {
+            LOG.warn("Method isUserInRole() is not implemented in HttpServletRequestWrapper, yet!");
+        }
         return false;
     }
 
@@ -289,8 +296,8 @@ public class HttpServletRequestWrapper extends ServletRequestWrapper implements
         /*
          * First look-up HttpSessionManagement if a session already exists
          */
-        final Cookie jsessionIdCookie = ajpRequestHandler.getHttpSessionCookie();
-        final String httpSessionId = jsessionIdCookie.getValue();
+        final Cookie sessionCookie = ajpRequestHandler.getHttpSessionCookie();
+        final String httpSessionId = sessionCookie.getValue();
         final HttpSessionWrapper httpSession = HttpSessionManagement.getHttpSession(httpSessionId);
         if (httpSession != null) {
             if (!HttpSessionManagement.isHttpSessionExpired(httpSession)) {
@@ -323,25 +330,25 @@ public class HttpServletRequestWrapper extends ServletRequestWrapper implements
             /*
              * Add JSESSIONID cookie
              */
-            configureCookie(jsessionIdCookie);
-            ajpRequestHandler.getServletResponse().addCookie(jsessionIdCookie);
+            configureCookie(sessionCookie);
+            ajpRequestHandler.getServletResponse().addCookie(sessionCookie);
         }
         return session;
     }
 
     private static final String DEFAULT_PATH = "/";
 
-    private static void configureCookie(final Cookie jsessionIdCookie) {
-        jsessionIdCookie.setPath(DEFAULT_PATH);
-        final ConfigurationService configurationService = Services.getService(ConfigurationService.class);
+    private static void configureCookie(final Cookie sessionCookie) {
+        sessionCookie.setPath(DEFAULT_PATH);
+        final ConfigurationService configurationService = AJPv13ServiceRegistry.getInstance().getService(ConfigurationService.class);
         /*
          * Check if auto-login is enabled
          */
         if (null != configurationService && configurationService.getBoolProperty("com.openexchange.sessiond.autologin", false)) {
-            final int maxAge = ConfigTools.parseTimespanSecs(configurationService.getProperty("com.openexchange.cookie.ttl", "1W"));
-            jsessionIdCookie.setMaxAge(maxAge);
+            final int maxAge = (int) (ConfigTools.parseTimespan(configurationService.getProperty("com.openexchange.cookie.ttl", "1W")) / 1000);
+            sessionCookie.setMaxAge(maxAge);
         } else {
-            jsessionIdCookie.setMaxAge(-1); // cookies auto-expire
+            sessionCookie.setMaxAge(-1); // cookies auto-expire
         }
     }
 
@@ -395,7 +402,8 @@ public class HttpServletRequestWrapper extends ServletRequestWrapper implements
         }
     }
 
-    private final ServletContext getServletContext() {
+    @Override
+    public ServletContext getServletContext() {
         return ServletConfigLoader.getDefaultInstance().getContext(servletInstance.getClass().getCanonicalName(), servletPath);
     }
 
@@ -403,4 +411,29 @@ public class HttpServletRequestWrapper extends ServletRequestWrapper implements
         this.startTime = currentTimeMillis;
     }
 
+    @Override
+    public boolean authenticate(final HttpServletResponse response) throws IOException, ServletException {
+        return false;
+    }
+
+    @Override
+    public void login(final String username, final String password) throws ServletException {
+        // Empty
+    }
+
+    @Override
+    public void logout() throws ServletException {
+        // Empty
+    }
+
+    @Override
+    public Collection<Part> getParts() throws IOException, ServletException {
+        return null;
+    }
+
+    @Override
+    public Part getPart(final String name) throws IOException, ServletException {
+        return null;
+    }
+
 }
