@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2012 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2014 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -49,6 +49,7 @@
 
 package com.openexchange.ajp13.servlet.http;
 
+import java.text.MessageFormat;
 import java.util.ArrayList;
 import java.util.Enumeration;
 import java.util.Iterator;
@@ -59,7 +60,8 @@ import javax.servlet.http.HttpSession;
 import javax.servlet.http.HttpSessionBindingEvent;
 import javax.servlet.http.HttpSessionBindingListener;
 import javax.servlet.http.HttpSessionContext;
-import com.openexchange.ajp13.Services;
+import com.openexchange.ajp13.AJPv13ServiceRegistry;
+import com.openexchange.ajp13.servlet.http.HttpSessionWrapper;
 import com.openexchange.config.ConfigTools;
 import com.openexchange.config.ConfigurationService;
 import com.openexchange.java.HashKeyMap;
@@ -86,11 +88,11 @@ public class HttpSessionWrapper implements HttpSession {
             synchronized (HttpSessionWrapper.class) {
                 tmp = cookieTTL;
                 if (tmp < 0) {
-                    final ConfigurationService service = Services.getService(ConfigurationService.class);
+                    final ConfigurationService service = AJPv13ServiceRegistry.getInstance().getService(ConfigurationService.class);
                     if (null == service) {
-                        tmp = ConfigTools.parseTimespanSecs("1W");
+                        tmp = (int) ConfigTools.parseTimespan("1W");
                     } else {
-                        tmp = ConfigTools.parseTimespanSecs(service.getProperty("com.openexchange.cookie.ttl", "1W"));
+                        tmp = (int) (ConfigTools.parseTimespan(service.getProperty("com.openexchange.cookie.ttl", "1W")) / 1000);
                     }
                     cookieTTL = tmp;
                 }
@@ -142,7 +144,7 @@ public class HttpSessionWrapper implements HttpSession {
         /*
          * Max. inactive interval
          */
-        final ConfigurationService service = Services.getService(ConfigurationService.class);
+        final ConfigurationService service = AJPv13ServiceRegistry.getInstance().getService(ConfigurationService.class);
         maxInactiveIntervall = null == service ? 1800 : service.getIntProperty("com.openexchange.servlet.maxInactiveIntervall", 1800); // 30 Minutes
         /*
          * Initialize other stuff
@@ -169,7 +171,7 @@ public class HttpSessionWrapper implements HttpSession {
     }
 
     @Override
-    public Enumeration<?> getAttributeNames() {
+    public Enumeration<String> getAttributeNames() {
         return new IteratorEnumeration(attributes.keySet().iterator());
     }
 
@@ -293,18 +295,18 @@ public class HttpSessionWrapper implements HttpSession {
     public void setMaxInactiveInterval(final int maxInactiveIntervall) {
         final int cookieTTL = getCookieTTL();
         if (maxInactiveIntervall < 0 || maxInactiveIntervall > cookieTTL) {
-            LOG.warn("Specified maxInactiveIntervall is negative or exceeds max. cookie time-to-live. Using max. cookie time-to-live: {}seconds", cookieTTL);
+            LOG.warn(MessageFormat.format("Specified maxInactiveIntervall is negative or exceeds max. cookie time-to-live. Using max. cookie time-to-live: {0}seconds", cookieTTL));
             this.maxInactiveIntervall = cookieTTL;
         } else {
             this.maxInactiveIntervall = maxInactiveIntervall;
         }
     }
 
-    private static class IteratorEnumeration implements Enumeration<Object> {
+    private static class IteratorEnumeration implements Enumeration<String> {
 
-        private final Iterator<?> iter;
+        private final Iterator<String> iter;
 
-        public IteratorEnumeration(final Iterator<?> iter) {
+        public IteratorEnumeration(final Iterator<String> iter) {
             this.iter = iter;
         }
 
@@ -314,7 +316,7 @@ public class HttpSessionWrapper implements HttpSession {
         }
 
         @Override
-        public Object nextElement() {
+        public String nextElement() {
             return iter.next();
         }
     }
