@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2012 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2014 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -378,7 +378,7 @@ public final class ServletConfigLoader {
             if (!clazzGuardian.contains(clazz) && null != directory) {
                 props = loadProperties(new File(
                     directory,
-                    new com.openexchange.java.StringAllocator(32).append(clazz).append('.').append(FILEEXT_PROPERTIES).toString()));
+                    new StringBuilder(32).append(clazz).append('.').append(FILEEXT_PROPERTIES).toString()));
                 if (null == props) {
                     clazzGuardian.add(clazz);
                 } else {
@@ -422,7 +422,7 @@ public final class ServletConfigLoader {
             if (!clazzGuardian.contains(clazz) && null != directory) {
                 props = loadProperties(new File(
                     directory,
-                    new com.openexchange.java.StringAllocator(32).append(clazz).append('.').append(FILEEXT_PROPERTIES).toString()));
+                    new StringBuilder(32).append(clazz).append('.').append(FILEEXT_PROPERTIES).toString()));
                 if (null == props) {
                     clazzGuardian.add(clazz);
                 } else {
